@@ -2,9 +2,10 @@
  * 
  * fitqtl_imp.c
  *
- * copyright (c) 2002-3, Hao Wu, The Jackson Laboratory
+ * copyright (c) 2002-5, Hao Wu, The Jackson Laboratory
+ *     Modified by Karl W. Broman to get estimates of QTL effects
  *
- * last modified Dec, 2003
+ * last modified Aug, 2005
  * first written Apr, 2002
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -27,16 +28,19 @@
 #include "util.h"
 #include "fitqtl_imp.h"
 #define TOL 1e-12
+#define IDXINTQ 15  /* maximum no. QTLs in an interaction */
+#define IDXINTC 10  /* maximum no. covariates in an interaction */
 
 void R_fitqtl_imp(int *n_ind, int *n_qtl, int *n_gen, int *n_draws,
 		  int *draws, int *n_cov, double *cov, int *model, 
-		  int *n_int, double *pheno,
+		  int *n_int, double *pheno, int *get_ests,
 		  /* return variables */
-		  double *lod, int *df)
+		  double *lod, int *df, double *ests, double *ests_covar,
+		  double *design_mat)
 {
   /* reorganize draws */
   int ***Draws;
-  double **Cov; 
+  double **Cov;
 
   reorg_draws(*n_ind, *n_qtl, *n_draws, draws, &Draws);
 
@@ -45,13 +49,14 @@ void R_fitqtl_imp(int *n_ind, int *n_qtl, int *n_gen, int *n_draws,
   if(*n_cov != 0) reorg_errlod(*n_ind, *n_cov, cov, &Cov);
 
   fitqtl_imp(*n_ind, *n_qtl, n_gen, *n_draws, Draws,
-              Cov, *n_cov, model, *n_int, pheno, lod, df);
+	     Cov, *n_cov, model, *n_int, pheno, *get_ests, lod, df,
+	     ests, ests_covar, design_mat);
 }
 
 
 /**********************************************************************
  * 
- * fitqlt_imp
+ * fitqtl_imp
  *
  * Performs general genome scan using the pseudomarker algorithm 
  * (imputation) method of Sen and Churchill (2001).
@@ -77,22 +82,35 @@ void R_fitqtl_imp(int *n_ind, int *n_qtl, int *n_gen, int *n_draws,
  *
  * pheno        Phenotype data, as a vector
  *
+ * get_ests     0/1: If 1, return estimated effects and their variances
+ *
  * lod          Return LOD score
  *
  * df           Return degree of freedom
  *
+ * ests         Return ests (vector of length sizefull)
+ *
+ * ests_covar   Return covariance matrix of ests (sizefull^2 matrix)
+ *
  **********************************************************************/
 
 void fitqtl_imp(int n_ind, int n_qtl, int *n_gen, int n_draws, 
 		int ***Draws, double **Cov, int n_cov, 
-		int *model, int n_int, double *pheno, double *lod, int *df) 
+		int *model, int n_int, double *pheno, int get_ests,
+		double *lod, int *df, double *ests, double *ests_covar,
+		double *design_mat) 
 {
 
   /* create local variables */
-  int i, j, n_qc, itmp; /* loop variants and temp variables */
+  int i, j, ii, jj, n_qc, itmp; /* loop variants and temp variables */
   double tol, lrss, lrss0, *LOD_array;
-  double *dwork;
-  int *iwork, sizefull;
+  double *the_ests, *the_covar, **TheEsts, ***TheCovar;
+  double *dwork, **Ests_covar, tot_wt, *wts;
+  double **Covar_mean, **Mean_covar, *mean_ests; /* for ests and cov matrix */
+  int *iwork, sizefull, n_trim, *index;
+
+  /* number to trim from each end of the imputations */
+  n_trim = (int) floor( 0.5*log(n_draws)/log(2.0) );
 
   /* initialization */
   sizefull = 1;
@@ -112,6 +130,24 @@ void fitqtl_imp(int n_ind, int n_qtl, int *n_gen, int n_draws,
     sizefull += itmp; 
   }
 
+  /* reorganize Ests_covar for easy use later */
+  /* and make space for estimates and covariance matrix */
+  if(get_ests) {
+    reorg_errlod(sizefull, sizefull, ests_covar, &Ests_covar);
+
+    allocate_double(sizefull*n_draws, &the_ests);
+    allocate_double(sizefull*sizefull*n_draws, &the_covar);
+
+    /* I need to save all of the estimates and covariance matrices */
+    reorg_errlod(sizefull, n_draws, the_ests, &TheEsts);
+    reorg_genoprob(sizefull, sizefull, n_draws, the_covar, &TheCovar);
+
+    allocate_dmatrix(sizefull, sizefull, &Mean_covar);
+    allocate_dmatrix(sizefull, sizefull, &Covar_mean);
+    allocate_double(sizefull, &mean_ests);
+    allocate_double(n_draws, &wts);
+  }
+
   /* allocate memory for working arrays, total memory is
      sizefull*n_ind+2*n_ind+4*sizefull for double array, 
      and sizefull for integer array.
@@ -119,6 +155,7 @@ void fitqtl_imp(int n_ind, int n_qtl, int *n_gen, int n_draws,
   dwork = (double *)R_alloc(sizefull*n_ind+2*n_ind+4*sizefull,
 			    sizeof(double));
   iwork = (int *)R_alloc(sizefull, sizeof(int));
+  index = (int *)R_alloc(n_draws, sizeof(int));
   LOD_array = (double *)R_alloc(n_draws, sizeof(double));
 
 
@@ -129,18 +166,85 @@ void fitqtl_imp(int n_ind, int n_qtl, int *n_gen, int n_draws,
   for(i=0; i<n_draws; i++) {
     /* calculate alternative model RSS */
     lrss = log10( galtRss(pheno, n_ind, n_gen, n_qtl, Draws[i], 
-			  Cov, n_cov, model, n_int, dwork, iwork, sizefull) );
+			  Cov, n_cov, model, n_int, dwork, iwork, sizefull,
+			  get_ests, ests, Ests_covar, (i==0), design_mat) );
 
     /* calculate the LOD score in this imputation */
     LOD_array[i] = (double)n_ind/2.0*(lrss0-lrss);
+
+    /* if getting estimates, calculate the weights */
+    if(get_ests) { 
+      wts[i] = LOD_array[i]*log(10.0);
+      if(i==0) tot_wt = wts[i];
+      else tot_wt = addlog(tot_wt, wts[i]);
+      
+      for(ii=0; ii<sizefull; ii++) {
+	TheEsts[i][ii] = ests[ii];
+	for(jj=ii; jj<sizefull; jj++) 
+	  TheCovar[i][ii][jj] = Ests_covar[ii][jj];
+      }
+    }
+
   } /* end loop over imputations */
 
+  /* sort the lod scores, and trim the weights */
+  if(get_ests) {
+    for(i=0; i<n_draws; i++) {
+      index[i] = i;
+      wts[i] = exp(wts[i]-tot_wt);
+    }
+
+    rsort_with_index(LOD_array, index, n_draws);
+
+    for(i=0; i<n_trim; i++) 
+      wts[index[i]] = wts[index[n_draws-i-1]] = 0.0;
+
+    /* re-scale wts */
+    tot_wt = 0.0;
+    for(i=0; i<n_draws; i++) tot_wt += wts[i];
+    for(i=0; i<n_draws; i++) wts[i] /= tot_wt;
+  } 
+
   /* calculate the result LOD score */
   *lod = wtaverage(LOD_array, n_draws);
 
   /* degree of freedom equals to the number of columns of x minus 1 (mean) */
   *df = sizefull - 1;
 
+  /* get means and variances and covariances of estimates */
+  if(get_ests) { 
+    for(i=0; i<n_draws; i++) {
+      if(i==0) {
+	for(ii=0; ii<sizefull; ii++) {
+	  mean_ests[ii] = TheEsts[i][ii] * wts[i];
+	  for(jj=ii; jj<sizefull; jj++) {
+	    Mean_covar[ii][jj] = TheCovar[i][ii][jj] * wts[i];
+	    Covar_mean[ii][jj] = 0.0;
+	  }
+	}
+      }
+      else {
+	for(ii=0; ii<sizefull; ii++) {
+	  mean_ests[ii] += TheEsts[i][ii]*wts[i];
+	  for(jj=ii; jj<sizefull; jj++) {
+	    Mean_covar[ii][jj] += TheCovar[i][ii][jj]*wts[i];
+	    Covar_mean[ii][jj] += (TheEsts[i][ii]-TheEsts[0][ii])*
+	      (TheEsts[i][jj]-TheEsts[0][jj])*wts[i];
+	  }
+	}
+      }
+    }
+      
+    for(i=0; i<sizefull; i++) {
+      ests[i] = mean_ests[i];
+      for(j=i; j<sizefull; j++) {
+	Covar_mean[i][j] = (Covar_mean[i][j] - (mean_ests[i]-TheEsts[0][i])*
+			    (mean_ests[j]-TheEsts[0][j]))*(double)n_draws/(double)(n_draws-1);
+	Ests_covar[i][j] = Ests_covar[j][i] = Mean_covar[i][j] + Covar_mean[i][j];
+      }
+    }
+  } /* done getting estimates */
+
 }
 
 
@@ -173,16 +277,18 @@ double nullRss0(double *pheno, int n_ind)
 /* galtRss - calculate RSS for full model in general scan */
 double galtRss(double *pheno, int n_ind, int *n_gen, int n_qtl, 
 	       int **Draws, double **Cov, int n_cov, int *model, 
-	       int n_int, double *dwork, int *iwork, int sizefull) 
+	       int n_int, double *dwork, int *iwork, int sizefull,
+	       int get_ests, double *ests, double **Ests_covar,
+	       int save_design, double *designmat) 
 {
   /* local variables */
-  int i, j, k, kk, *jpvt, ny, idx_col, n_qc, itmp1, itmp2, n_int_col, tmp_idx;
-  double *work, *x, *qty, *qraux, *coef, *resid, tol;
+  int i, j, k, kk, *jpvt, ny, idx_col, n_qc, itmp1, itmp2, n_int_col, tmp_idx, job;
+  double *work, *x, *qty, *qraux, *coef, *resid, tol, sigmasq;
   /* The following vars are used for parsing model. 
      the dimension of idx_int_q and idx_int_c are set to be arbitrary number
-     for the ease of programming. But I think 15 and 10 are big enought. 
+     for the ease of programming. But I think 15 and 10 are big enough. 
      Is there any body want to try a 16 way interaction? */
-  int n_int_q, n_int_c, idx_int_q[15], idx_int_c[10]; 
+  int n_int_q, n_int_c, idx_int_q[IDXINTQ], idx_int_c[IDXINTC]; 
   /* return variable */
   double rss_full;
 
@@ -205,6 +311,10 @@ double galtRss(double *pheno, int n_ind, int *n_gen, int n_qtl,
   work = qraux + sizefull; 
   /* integer array */
   jpvt = iwork;
+  /* make jpvt = numbers 0, 1, ..., (sizefull-1) */
+  /*      jpvt keeps track of any permutation of X columns */
+  for(i=0; i<sizefull; i++) 
+    jpvt[i] = i;
 
   /******************************************************
    The following part will construct the design matrix x 
@@ -315,6 +425,11 @@ double galtRss(double *pheno, int n_ind, int *n_gen, int n_qtl,
   } /* finish the loop for interaction */
   /* finish design matrix construction */
 
+  /* save design matrix */
+  if(save_design) {
+    for(i=0; i<n_ind*sizefull; i++) 
+      designmat[i] = x[i];
+  }
 
   /* call dqrls to fit regression model */
   F77_CALL(dqrls)(x, &n_ind, &sizefull, pheno, &ny, &tol, coef, resid,
@@ -324,6 +439,28 @@ double galtRss(double *pheno, int n_ind, int *n_gen, int n_qtl,
   for(i=0; i<n_ind; i++)
     rss_full += resid[i]*resid[i];
 
+
+  if(get_ests) { /* get the estimates and their covariance matrix */
+    /* get ests; need to permute back */
+    for(i=0; i<k; i++) 
+      ests[jpvt[i]] = coef[i];
+    for(i=k; i<sizefull; i++)
+      ests[jpvt[i]] = 0.0;
+    
+    /* get covariance matrix: dpodi to get (X'X)^-1; re-sort; multiply by sigma_hat^2 */
+    job = 1; /* indicates to dpodi to get inverse and not determinant */
+    F77_CALL(dpodi)(x, &n_ind, &sizefull, work, &job);
+
+    sigmasq = rss_full/(double)(n_ind-sizefull);
+    for(i=0; i<k; i++) 
+      for(j=i; j<k; j++) 
+	Ests_covar[jpvt[i]][jpvt[j]] = Ests_covar[jpvt[j]][jpvt[i]] = 
+	  x[j*n_ind+i] *sigmasq;
+    for(i=k; i<sizefull; i++)
+      for(j=0; j<k; j++)
+	Ests_covar[jpvt[i]][j] = Ests_covar[j][jpvt[i]] = 0.0;
+  }
+
   return(rss_full);
 }
 
