@@ -2,9 +2,9 @@
  * 
  * scantwo_hk.h
  *
- * copyright (c) 2001, Karl W Broman, Johns Hopkins University
+ * copyright (c) 2001-2, Karl W Broman, Johns Hopkins University
  *
- * last modified Nov, 2001
+ * last modified Oct, 2002
  * first written Nov, 2001
  *
  * Licensed under the GNU General Public License version 2 (June, 1991)
@@ -32,7 +32,8 @@ void R_scantwo_1chr_hk(int *n_ind, int *n_pos, int *n_gen,
 		       double *genoprob, double *pairprob, 
 		       double *addcov, int *n_addcov, 
 		       double *intcov, int *n_intcov, 
-		       double *pheno, double *result);
+		       double *pheno, double *weights, 
+		       double *result);
 
 /**********************************************************************
  * 
@@ -66,9 +67,11 @@ void R_scantwo_1chr_hk(int *n_ind, int *n_pos, int *n_gen,
  *
  * pheno        Phenotype data, as a vector
  *
- * Result       Result matrix of size [n_pos x n_pos]; the upper 
- *              triangle (row < col) contains the joint LODs while 
- *              the lower triangle (row > col) contains the LODs for 
+ * weights      Vector of positive weights, of length n_ind
+ *
+ * Result       Result matrix of size [n_pos x n_pos]; the lower
+ *              triangle (row > col) contains the joint LODs while 
+ *              the upper triangle (row < col) contains the LODs for 
  *              testing epistasis.
  *              Note: indexed as Result[col][row]
  *
@@ -77,7 +80,7 @@ void R_scantwo_1chr_hk(int *n_ind, int *n_pos, int *n_gen,
 void scantwo_1chr_hk(int n_ind, int n_pos, int n_gen, double ***Genoprob,
 		     double *****Pairprob, double **Addcov, int n_addcov, 
 		     double **Intcov, int n_intcov, double *pheno, 
-		     double **Result);
+		     double *weights, double **Result);
 
 /**********************************************************************
  * 
@@ -93,8 +96,8 @@ void R_scantwo_2chr_hk(int *n_ind, int *n_pos1, int *n_pos2,
 		       double *genoprob1, double *genoprob2,
 		       double *addcov, int *n_addcov, 
 		       double *intcov, int *n_intcov, 
-		       double *pheno, double *result_full,
-		       double *result_int);
+		       double *pheno, double *weights,
+		       double *result_full, double *result_int);
 
 /**********************************************************************
  * 
@@ -111,9 +114,9 @@ void R_scantwo_2chr_hk(int *n_ind, int *n_pos1, int *n_pos2,
  *
  * n_pos2       Number of marker positions on second chromosome
  *
- * n_gen1       Number of different genotypes on first chromosome
+ * n_gen1       Number of different genotypes for first chromosome
  *
- * n_gen2       Number of different genotypes on second chromosome
+ * n_gen2       Number of different genotypes for second chromosome
  *
  * Genoprob1    Array of conditional genotype probs for 1st chr
  *              Indexed as Genoprob[gen][pos][ind]
@@ -130,20 +133,24 @@ void R_scantwo_2chr_hk(int *n_ind, int *n_pos1, int *n_pos2,
  *
  * pheno        Phenotype data, as a vector
  *
+ * weights      Vector of positive weights, of length n_ind
+ *
  * Result_full  Result matrix of size [n_pos1 x n_pos2]
  *              containing the joint LODs
  *              Note: indexed as Result[pos2][pos1]
  *
- * Result_int   Result matrix of size [n_pos1 x n_pos2] 
+ * Result_int   Result matrix of size [n_pos2 x n_pos1] 
  *              containing the LODs testing interactions
+ *              also indexed as Result[pos2][pos1]
  *
  **********************************************************************/
 
 void scantwo_2chr_hk(int n_ind, int n_pos1, int n_pos2, int n_gen1, 
-		     int n_gen2,
-		     double ***Genoprob1, double ***Genoprob2, 
+		     int n_gen2, double ***Genoprob1, 
+		     double ***Genoprob2, 
 		     double **Addcov, int n_addcov, 
 		     double **Intcov, int n_intcov, double *pheno, 
+		     double *weights,
 		     double **Result_full, double **Result_int);
 
 /* end of scantwo_hk.h */
